/*
 * Hangman application
 * BME MIT, IRF course
 * https://www.inf.mit.bme.hu/edu/bsc/irf
 */
package hangman;

import java.io.IOException;
import java.net.Socket;

/**
 *
 * Hangman game session.
 *
 * @author Tamas Toth
 */
public class GameSession extends ServerThread {

    public GameSession(Socket socket) throws IOException {
        super(socket);
    }

    @Override
    public void mainMethod() {
        try {

            //initialize game session
            int round = 0;
            int score;

            Puzzle puzzle;

            boolean shouldExit = false;

            //welocme message
            writeLine("-------------------");
            writeLine("Simple Hangman v0.1");
            writeLine("-------------------");

            //main game loop
            while (!shouldExit) {
                //initialize current round
                round++;
                score = 1000;

                boolean vowelPowerupUsed = false;

                puzzle = Glossary.getInstance().createPuzzle();

                //round header
                writeLine();
                writeLine("Round " + round + ".");
                writeLine();
                writeLine("You have a new puzzle to solve!");
                writeLine("Category: " + puzzle.getCategory());

                //round loop
                while (!shouldExit && !puzzle.isSolved()) {

                    //write out menu options
                    writeLine();
                    writeLine("-------------------");
                    writeLine();
                    writeLine("Score: " + score);
                    writeLine();
                    writeLine(puzzle.toString());
                    writeLine();
                    writeLine("[1] Ask consonant");
                    writeLine("[2] Ask vowel [powerup]" + (vowelPowerupUsed ? " [used]" : ""));
                    writeLine("[3] Solve puzzle");
                    writeLine("[4] Exit game");
                    writeLine();

                    boolean validOption = false;
                    String line;
                    int option = 0;

                    //read an option
                    while (!validOption) {
                        write("My choice: ");
                        line = readLine();

                        try {
                            option = Integer.parseInt(line);
                        } catch (NumberFormatException e) {
                            writeLine("You should enter a number!");
                            continue;
                        }

                        if (option < 1 || option > 4) {
                            writeLine("Please select an option from the above!");
                            continue;
                        }

                        validOption = true;
                    }

                    writeLine();

                    boolean validInput = false;

                    //read the relevant input
                    switch (option) {
                        //option 1: ask consonant
                        case 1:
                            while (!validInput) {
                                write("Please enter a consonant: ");
                                line = readLine();

                                //input should be a single letter
                                if (line.length() != 1) {
                                    writeLine("Please enter a single letter!");
                                    continue;
                                }

                                char c = line.charAt(0);

                                //the letter should be a consonant
                                if ("bcdfghjklmnpqrstvwxyzBCDFGHJKLMNPQRSTVWXYZ".indexOf(c) < 0) {
                                    writeLine("Letter " + c + " is not a consonant!");
                                    continue;
                                }

                                //the letter should not have been already asked
                                if (puzzle.isAsked(c)) {
                                    writeLine("You have already asked for letter " + c + "!");
                                    break;
                                }

                                validInput = true;

                                //ask letter
                                puzzle.Ask(c);
                                if (puzzle.isRevealed(c)) {
                                    writeLine("Great success!");
                                } else {
                                    writeLine("Sorry, bad letter. Score decreased by 50.");
                                    score -= 50;
                                }
                            }
                            break;
                        //option 2: ask vowel
                        case 2:
                            //powerup should be available
                            if (vowelPowerupUsed) {
                                write("You have already used that powerup!");
                                continue;
                            }

                            while (!validInput) {
                                write("Please enter a vowel: ");
                                line = readLine();

                                //input should be a single letter      
                                if (line.length() != 1) {
                                    writeLine("Please enter a single letter!");
                                    continue;
                                }

                                char c = line.charAt(0);

                                //the letter should be a vowel
                                if ("aeiouAEIOU".indexOf(c) < 0) {
                                    writeLine("Letter " + c + " is not a vowel!");
                                    continue;
                                }

                                //the letter should not have been already asked
                                if (puzzle.isAsked(c)) {
                                    writeLine("You have already asked for letter " + c + "!");
                                    break;
                                }

                                validInput = true;
                                vowelPowerupUsed = true;

                                //ask letter
                                puzzle.Ask(c);
                                if (puzzle.isRevealed(c)) {
                                    writeLine("Great success!");
                                } else {
                                    writeLine("Sorry, bad letter. Score decreased by 100.");
                                    score -= 100;
                                }
                            }
                            break;
                        //option 3: guess
                        case 3:
                            write("Your guess: ");
                            line = readLine();

                            //guess
                            puzzle.Guess(line);
                            if (!puzzle.isSolved()) {
                                writeLine("Sorry, bad guess. Score decreased by 250.");
                                score -= 250;
                            }

                            break;
                        //option 4: exit
                        case 4:
                            writeLine();
                            writeLine("Thanks for playing. Bye!");
                            shouldExit = true;
                            break;
                    }
                }
                if (puzzle.isSolved()) {
                    writeLine();
                    writeLine("You have managed to solve the puzzle. Happy times!");
                    writeLine("Your final socre is: " + score + ".");
                    writeLine();
                    writeLine("-------------------");
                }
            }
        } catch (IOException e) {
            System.err.println("An error occured accessing the client, execution stops.");
        }
    }
}