/*
 * Hangman application
 * BME MIT, IRF course
 * https://www.inf.mit.bme.hu/edu/bsc/irf
 */
package hangman;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Singleton class for parsing a .csv file that contains all puzzles.
 *
 * @author Tamas Toth
 */
public class Glossary {
    
    private static final String filename = "/glossary.csv";
    
    private static Glossary instance;
    private List<String> riddles;
    private List<String> categories;

    protected Glossary() {
        riddles = new ArrayList<>();
        categories = new ArrayList<>();

        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream(filename)));
            String line;
            while ((line = br.readLine()) != null) {
                StringTokenizer tokenizer = new StringTokenizer(line, ";");

                String riddle = tokenizer.nextToken();
                String category = tokenizer.nextToken();

                riddles.add(riddle);
                categories.add(category);
            }
        } catch (Exception e) {
            System.err.println("Could not open file.");
        }
    }

    public static Glossary getInstance() {
        if (instance == null) {
            instance = new Glossary();
        }
        return instance;
    }

    /**
     * Instantiates a random puzzle.
     *
     * @return A random puzzle.
     */
    public Puzzle createPuzzle() {
        int i = Math.round((float) Math.random() * (riddles.size() - 1));
        return new Puzzle(riddles.get(i), categories.get(i));
    }
}