/*
 * Hangman application
 * BME MIT, IRF course
 * https://www.inf.mit.bme.hu/edu/bsc/irf
 */
package hangman;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.Socket;

/**
 * Abstract class that can be used in a thread for simple TCP/IP communication.
 *
 * @author Tamas Toth
 */
public abstract class ServerThread implements Runnable {

    private Socket socket;
    public BufferedReader in;
    public DataOutputStream out;

    public ServerThread(Socket socket) throws IOException {
        this.socket = socket;
        in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        out = new DataOutputStream(socket.getOutputStream());
    }

    protected void write(String line) throws IOException {
        out.writeChars(line);

    }

    protected void writeLine(String line) throws IOException {
        out.writeChars(line + "\n\r");
    }

    protected void writeLine() throws IOException {
        out.writeChars("\n\r");
    }

    protected String readLine() throws IOException {
        return in.readLine();

    }

    @Override
    /**
     * The run() method is implemented as a template method to ensure that the
     * socket is closed after the mainMethod() has ended.
     *
     */
    public void run() {
        System.out.println("Connection established with " + socket.getInetAddress() + ":" + socket.getPort());

        //strart the main function
        mainMethod();

        //close the socket
        try {
            socket.close();
            System.out.println("Connection with " + socket.getInetAddress() + ":" + socket.getPort() + " closed");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * The main function.
     */
    protected abstract void mainMethod();
}
